#!/bin/sh

# Shell script for invoking the wing IDE.  If the first argument is --winghome
# then the directory wing is installed in is echoed to stdout and the script exits.
# Other arguments are:
#  --verbose     Print debugging output
#  --silent      Run silently
#  --system-gtk  Use system-wide version of gtk, rather than Wing's copy
#  --private-gtk Use Wing's copy of gtk rather than the system version
#
# To develop and run from sources, set WINGHOME and WINGPYTHON
# 
# Copyright (c) 2000-2007, Archaeopteryx Software, Inc.  All rights reserved.
#
# Written by Stephan R.A. Deibel and John P. Ehresman

# Find WINGHOME unless caller has passed it in
if [ "${WINGHOME}" = "" ]; then
    WINGHOME=`dirname "$0"`; export WINGHOME
fi

# Set up library paths on OS X (Linux instead loads libs directly in wing.py)
UNAME=`uname`
if [ ${UNAME} = 'Darwin' ]; then
    # Add all runtime-*/lib subdirectories as library dirs
    ALL_RUNTIME_NAMES="python2.7 qt pyside-py2.7"
    LIBDIRS=""
    for NAME in ${ALL_RUNTIME_NAMES}
    do
        if [ "${LIBDIRS}" = "" ]; then
            LIBDIRS="${WINGHOME}/bin/runtime-${NAME}/lib"
        else
            LIBDIRS="${WINGHOME}/bin/runtime-${NAME}/lib:${LIBDIRS}"
        fi
    done

    # Add Steam library dir, if present
    STEAM_LIBDIR="${WINGHOME}/../../../shared/steam/lib/osx32"
    if [ ! -d "${STEAM_LIBDIR}" ]; then
        STEAM_LIBDIR="${WINGHOME}/external/steam/lib/osx32"
    fi
    if [ -d "${STEAM_LIBDIR}" ]; then
        LIBDIRS="${LIBDIRS}:${STEAM_LIBDIR}"
    fi

    # Set up OS X library path
    if [ "$DYLD_LIBRARY_PATH" = "" ]; then
        WING_ORIG_DYLD_LIBRARY_PATH=__undefined__
        DYLD_LIBRARY_PATH="${LIBDIRS}"
    else
        WING_ORIG_DYLD_LIBRARY_PATH=$DYLD_LIBRARY_PATH
        DYLD_LIBRARY_PATH="${LIBDIRS}:$DYLD_LIBRARY_PATH"
    fi
    export DYLD_LIBRARY_PATH
    export WING_ORIG_DYLD_LIBRARY_PATH

    if [ "$DYLD_FRAMEWORK_PATH" = "" ]; then
        WING_ORIG_DYLD_FRAMEWORK_PATH=__undefined__
        DYLD_FRAMEWORK_PATH="${LIBDIRS}"
    else
        WING_ORIG_DYLD_FRAMEWORK_PATH=$DYLD_FRAMEWORK_PATH
        DYLD_FRAMEWORK_PATH="${LIBDIRS}:$DYLD_FRAMEWORK_PATH"
    fi
    export DYLD_FRAMEWORK_PATH
    export WING_ORIG_DYLD_FRAMEWORK_PATH
    
    # Use qt debug libraries if the QtCore debug libary exists; WING_ORIG_* isn't
    # set because this is only for development
    if [ -f "${WINGHOME}/bin/runtime-qt/lib/QtCore.framework/QtCore_debug" ]; then
        DYLD_IMAGE_SUFFIX=_debug
        export DYLD_IMAGE_SUFFIX
    fi
fi

# Check if user requested debug output or we're running from source in which
# case we also want debug output (but can use --silent to force squelching of
# output also when running from source)
if [ "$1" = "--verbose" ]; then
  shift
  DEBUGARG=
elif [ "$1" = "--silent" ]; then
  shift
  DEBUGARG=--squelch-output
else
  DEBUGARG=--squelch-output
fi

# Location of Python interpreter to use, if not already set
if [ "${WINGPYTHON}" = "" ]; then
  if [ -x "${WINGHOME}/wing-internal-python" ]; then
      WINGPYTHON="${WINGHOME}/wing-internal-python"
  elif [ -x "${WINGHOME}/../MacOS/wing-internal-python" ]; then
      WINGPYTHON="${WINGHOME}/../MacOS/wing-internal-python"
  elif [ -x "${WINGHOME}/bin/PyCore/python" ]; then
      WINGPYTHON="${WINGHOME}/bin/PyCore/python"
  else
      WINGPYTHON="${WINGHOME}/bin/runtime-python2.7/bin/python2.7"
  fi
fi

# Just echo home and exit if requested
if [ "$1" = "--winghome" ]; then
    echo ${WINGHOME}
    exit 0
fi

# Run Python optimized and ignore environment
PYTHON_ARGS="-OSE"

# Run the IDE
if [ -f "${WINGHOME}/bin/wing.py" ]; then
    exec "${WINGPYTHON}" $PYTHON_ARGS "${WINGHOME}/bin/wing.py" $DEBUGARG "$@"
elif [ -f "${WINGHOME}/src/wing.py" ]; then
    exec "${WINGPYTHON}" $PYTHON_ARGS "${WINGHOME}/src/wing.py" $DEBUGARG "$@"
elif [ "$DEBUGARG" = "" ]; then
    echo "wing: Error: wing.py not found.  Please check your Wing IDE installation"
fi
